@@ -29,13 +29,12 @@ import re
 import textwrap
 import typing as t
 
-from nr.util.scanner import Scanner
+from nr.util.iter import SequenceWalker
 
 from docspec import (
-  ApiObject,
   Argument,
   Class,
-  Data,
+  Variable,
   Docstring,
   Decoration,
   Function,
@@ -49,9 +48,6 @@ from lib2to3.pgen2.parse import ParseError
 from lib2to3.pygram import python_symbols as syms
 from lib2to3.pytree import Leaf, Node
 
-_REVERSE_SYMS = {v: k for k, v in vars(syms).items() if isinstance(v, int)}
-_REVERSE_TOKEN = {v: k for k, v in vars(token).items() if isinstance(v, int)}
-
 
 def dedent_docstring(s):
   lines = s.split('\n')
@@ -183,12 +179,12 @@ class Parser:
       if node.type == syms.dotted_as_name:  # example: urllib.request as r
         target = self.name_to_string(node.children[0])
         name = self.name_to_string(node.children[2])
-        return Indirection(name, self.location_from(node), None, target)
+        return Indirection(self.location_from(node),name, None, target)
       elif node.type == syms.dotted_name:  # example os.path
         name = self.name_to_string(node)
-        return Indirection(name.split('.')[-1], self.location_from(node), None, name)
+        return Indirection(self.location_from(node), name.split('.')[-1], None, name)
       elif isinstance(node, Leaf):
-        return Indirection(node.value, self.location_from(node), None, node.value)
+        return Indirection(self.location_from(node), node.value, None, node.value)
       else:
         raise RuntimeError(f'cannot handle {node!r}')
 
@@ -196,12 +192,12 @@ class Parser:
       if node.type == syms.import_as_name:  # example: Widget as W
         target = self.name_to_string(node.children[0])
         name = self.name_to_string(node.children[2])
-        return Indirection(name, self.location_from(node), None, prefix + '.' + target)
+        return Indirection(self.location_from(node), name, None, prefix + '.' + target)
       elif isinstance(node, Leaf):  # example: Widget
         name = self.name_to_string(node)
         if not prefix.endswith('.'):
           prefix += '.'
-        return Indirection(name, self.location_from(node), None, prefix + name)
+        return Indirection(self.location_from(node), name, None, prefix + name)
       else:
         raise RuntimeError(f'cannot handle {node!r}')
 
@@ -235,7 +231,7 @@ class Parser:
       assert names
       for name in names:
         name = self.nodes_to_string([name])
-        data = Data(
+        data = Variable(
           name=name,
           location=self.location_from(stmt),
           docstring=docstring,
@@ -249,7 +245,7 @@ class Parser:
     assert node.children[0].value == '@'
     name = self.name_to_string(node.children[1])
     call_expr = self.nodes_to_string(node.children[2:]).strip()
-    return Decoration(name=name, args=call_expr or None)
+    return Decoration(location=self.location_from(node), name=name, args=call_expr or None)
 
   def parse_funcdef(self, parent, node, is_async, decorations):
     parameters = find(lambda x: x.type == syms.parameters, node.children)
@@ -271,7 +267,7 @@ class Parser:
       return_type=return_,
       decorations=decorations)
 
-  def parse_argument(self, node: t.Union[Leaf, Node, None], argtype: Argument.Type, scanner: 'Scanner[Leaf | Node]') -> Argument:
+  def parse_argument(self, node: t.Union[Leaf, Node, None], argtype: Argument.Type, scanner: 'SequenceWalker[Leaf | Node]') -> Argument:
     """
     Parses an argument from the AST. *node* must be the current node at
     the current position of the *scanner*. The scanner is used to extract
@@ -280,7 +276,7 @@ class Parser:
 
     def parse_annotated_name(node):
       if node.type == syms.tname:
-        scanner = Scanner(node.children)
+        scanner = SequenceWalker(node.children)
         name = scanner.current.value
         node = scanner.next()
         assert node.type == token.COLON, node.parent
@@ -294,6 +290,7 @@ class Parser:
       return (name, annotation)
 
     assert node is not None
+    location = self.location_from(node)
     name, annotation = parse_annotated_name(node)
     assert name not in '/*', repr(node)
 
@@ -304,7 +301,13 @@ class Parser:
       default = self.nodes_to_string([node])
       scanner.advance()
 
-    return Argument(name, argtype, None, annotation, default)
+    return Argument(
+      location=location,
+      name=name,
+      type=argtype,
+      datatype=annotation,
+      default_value=default,
+    )
 
   def parse_parameters(self, parameters):
     assert parameters.type == syms.parameters, parameters.type
@@ -316,18 +319,25 @@ class Parser:
       #   not get wrapped in a `typedargslist`, but in a single `tname`.
       tname = find(lambda x: x.type == syms.tname, parameters.children)
       if tname:
-        scanner = Scanner(parameters.children, parameters.children.index(tname))
+        scanner = SequenceWalker(parameters.children, parameters.children.index(tname))
         result.append(self.parse_argument(tname, Argument.Type.POSITIONAL, scanner))
       else:
         # This must be either ["(", ")"] or ["(", "argname", ")"].
         assert len(parameters.children) in (2, 3), parameters.children
         if len(parameters.children) == 3:
-          result.append(Argument(parameters.children[1].value, Argument.Type.POSITIONAL, None, None, None))
+          result.append(Argument(
+            location=self.location_from(parameters.children[1]),
+            name=parameters.children[1].value,
+            type=Argument.Type.POSITIONAL,
+            decorations=None,
+            datatype=None,
+            default_value=None,
+          ))
       return result
 
     argtype = Argument.Type.POSITIONAL
 
-    index = Scanner(arglist.children)
+    index = SequenceWalker(arglist.children)
     for node in index.safe_iter():
       node = index.current
 
@@ -376,7 +386,7 @@ class Parser:
   def parse_classdef_rawargs(self, classdef):
     metaclass = None
     bases = []
-    index = Scanner(classdef.children, 2)
+    index = SequenceWalker(classdef.children, 2)
     if index.current.type == token.LPAR:
       index.next()
       while index.current.type != token.RPAR:
@@ -424,7 +434,7 @@ class Parser:
           members = [members]
         for member in members:
           assert not isinstance(member, Module)
-          if metaclass is None and isinstance(member, Data) and \
+          if metaclass is None and isinstance(member, Variable) and \
               member.name == '__metaclass__':
             metaclass = member.value
           elif member:
@@ -538,11 +548,11 @@ class Parser:
         else:
           line = line[1:]
         lines.append(line.lstrip())
-      return Docstring('\n'.join(lines).strip(), location)
+      return Docstring(location, '\n'.join(lines).strip())
     if s.startswith('"""') or s.startswith("'''"):
-      return Docstring(dedent_docstring(s[3:-3]).strip(), location)
+      return Docstring(location, dedent_docstring(s[3:-3]).strip())
     if s.startswith('"') or s.startswith("'"):
-      return Docstring(dedent_docstring(s[1:-1]).strip(), location)
+      return Docstring(location, dedent_docstring(s[1:-1]).strip())
     return None
 
   def nodes_to_string(self, nodes):
